@@ -168,7 +168,7 @@ load_config (int check_syntax)
 	  /* We have
 	     domain <domain> server <host|ip>
 	     or
-	     domain <domain> broadcast */
+	     domain <domain> broadcast*/
 
 	  count = sscanf (cp, "domain %64s server %80s", tmpdomain,
 			  tmpserver);
@@ -215,6 +215,20 @@ load_config (int check_syntax)
 	      continue;
 	    }
 	}
+      else if (strncmp (cp, "broadcast", 9) == 0 &&
+	       (isspace ((int)cp[9]) || cp[9] == '\0'))
+	{
+	  /* We have
+	     broadcast  */
+
+	  if (debug_flag)
+	    log_msg (LOG_DEBUG, _("parsed broadcast"));
+	  if (add_server (domain, NULL, check_syntax))
+	    ++have_entries;
+	  else
+	    ++bad_entries;
+	  continue;
+	}
       if (check_syntax)
 	{
 	  fprintf (stderr, _("Entry \"%s\" is not valid!\n"), cp);
@@ -649,10 +663,10 @@ main (int argc, char **argv)
       if (local_only)
 	socket_address.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
       else
-	{
-	  socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
-	  socket_address.sin_port = htons (port);
-	}
+	socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
+
+      if (port >= 0)
+	socket_address.sin_port = htons (port);
 
       result = bind (sock, (struct sockaddr *) &socket_address,
 		     sizeof (socket_address));
@@ -702,10 +716,10 @@ main (int argc, char **argv)
       if (local_only)
 	socket_address.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
       else
-	{
-	  socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
-	  socket_address.sin_port = htons (port);
-	}
+	socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
+
+      if (port >= 0)
+	socket_address.sin_port = htons (port);
 
       result = bind (sock, (struct sockaddr *) &socket_address,
 		     sizeof (socket_address));
